@@ -14,10 +14,15 @@
 #include <execinfo.h>
 #include <csignal>
 #include <sys/poll.h>
+#ifdef __linux
 #include <sys/prctl.h>
+#include <sys/syscall.h>
+#include <sys/vfs.h>
+#elif defined IOS
+#import <Foundation/Foundation.h>
+#endif
 #include <sys/stat.h>
 #include <sys/uio.h>
-#include <sys/vfs.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -56,6 +61,8 @@
 #include "Log.hpp"
 #include "Util.hpp"
 
+using std::size_t;
+
 namespace Util
 {
     namespace rng
@@ -140,6 +147,7 @@ namespace Util
         return newTmp;
     }
 
+#ifndef MOBILEAPP
     int getProcessThreadCount()
     {
         DIR *fdDir = opendir("/proc/self/task");
@@ -232,6 +240,7 @@ namespace Util
             int ret = execvp(params[0], &params[0]);
             if (ret < 0)
                 std::cerr << "Failed to exec command '" << cmd << "' with error '" << strerror(errno) << "'\n";
+            Log::shutdown();
             _exit(42);
         }
         // else spawning process still
@@ -242,6 +251,7 @@ namespace Util
         }
         return pid;
     }
+#endif
 
     bool dataFromHexString(const std::string& hexString, std::vector<unsigned char>& data)
     {
@@ -285,9 +295,11 @@ namespace Util
         return std::getenv("DISPLAY") != nullptr;
     }
 
+#ifndef MOBILEAPP
+
     static const char *startsWith(const char *line, const char *tag)
     {
-        int len = strlen(tag);
+        size_t len = std::strlen(tag);
         if (!strncmp(line, tag, len))
         {
             while (!isdigit(line[len]) && line[len] != '\0')
@@ -456,6 +468,7 @@ namespace Util
         }
         return 0;
     }
+#endif
 
     std::string replace(std::string result, const std::string& a, const std::string& b)
     {
@@ -485,19 +498,24 @@ namespace Util
         return replace(r, "\n", " / ");
     }
 
-    static __thread char ThreadName[32];
+    static __thread char ThreadName[32] = {0};
 
     void setThreadName(const std::string& s)
     {
         strncpy(ThreadName, s.c_str(), 31);
         ThreadName[31] = '\0';
+#ifdef __linux
         if (prctl(PR_SET_NAME, reinterpret_cast<unsigned long>(s.c_str()), 0, 0, 0) != 0)
-            LOG_SYS("Cannot set thread name of " << getThreadId() << " (" <<
-                    std::this_thread::get_id() << ") to [" << s << "].");
+            LOG_SYS("Cannot set thread name of " << getThreadId() << " (" << std::hex <<
+                    std::this_thread::get_id() << std::dec << ") to [" << s << "].");
         else
-            LOG_INF("Thread " << getThreadId() << " (" <<
-                    std::this_thread::get_id() <<
-                    ") is now called [" << s << "].");
+            LOG_INF("Thread " << getThreadId() << " (" << std::hex <<
+                    std::this_thread::get_id() << std::dec << ") is now called [" << s << "].");
+#elif defined IOS
+        [[NSThread currentThread] setName:[NSString stringWithUTF8String:ThreadName]];
+        LOG_INF("Thread " << getThreadId() <<
+                ") is now called [" << s << "].");
+#endif
     }
 
     const char *getThreadName()
@@ -505,22 +523,36 @@ namespace Util
         // Main process and/or not set yet.
         if (ThreadName[0] == '\0')
         {
+#ifdef __linux
             if (prctl(PR_GET_NAME, reinterpret_cast<unsigned long>(ThreadName), 0, 0, 0) != 0)
-                ThreadName[0] = '\0';
+                strncpy(ThreadName, "<noid>", sizeof(ThreadName) - 1);
+#elif defined IOS
+            const char *const name = [[[NSThread currentThread] name] UTF8String];
+            strncpy(ThreadName, name, 31);
+            ThreadName[31] = '\0';
+#endif
         }
 
         // Avoid so many redundant system calls
         return ThreadName;
     }
 
-    static __thread pid_t ThreadTid;
+#ifdef __linux
+    static __thread pid_t ThreadTid = 0;
 
     pid_t getThreadId()
+#else
+    std::thread::id getThreadId()
+#endif
     {
         // Avoid so many redundant system calls
+#ifdef __linux
         if (!ThreadTid)
-            ThreadTid = syscall(SYS_gettid);
+            ThreadTid = ::syscall(SYS_gettid);
         return ThreadTid;
+#else
+        return std::this_thread::get_id();
+#endif
     }
 
     void getVersionInfo(std::string& version, std::string& hash)
@@ -581,6 +613,105 @@ namespace Util
 
         return true;
     }
+
+    /// Split a string in two at the delimeter and give the delimiter to the first.
+    static
+    std::pair<std::string, std::string> splitLast2(const char* s, const int length, const char delimeter = ' ')
+    {
+        if (s != nullptr && length > 0)
+        {
+            const int pos = getLastDelimiterPosition(s, length, delimeter);
+            if (pos < length)
+                return std::make_pair(std::string(s, pos + 1), std::string(s + pos + 1));
+        }
+
+        // Not found; return in first.
+        return std::make_pair(std::string(s, length), std::string());
+    }
+
+    std::tuple<std::string, std::string, std::string, std::string> splitUrl(const std::string& url)
+    {
+        // In case we have a URL that has parameters.
+        std::string base;
+        std::string params;
+        std::tie(base, params) = Util::split(url, '?', false);
+
+        std::string filename;
+        std::tie(base, filename) = Util::splitLast2(base.c_str(), base.size(), '/');
+        if (filename.empty())
+        {
+            // If no '/', then it's only filename.
+            std::swap(base, filename);
+        }
+
+        std::string ext;
+        std::tie(filename, ext) = Util::splitLast(filename, '.', false);
+
+        return std::make_tuple(base, filename, ext, params);
+    }
+
+    static std::map<std::string, std::string> AnonymizedStrings;
+    static std::atomic<unsigned> AnonymizationSalt(0);
+    static std::mutex AnonymizedMutex;
+
+    void mapAnonymized(const std::string& plain, const std::string& anonymized)
+    {
+        if (plain.empty() || anonymized.empty())
+            return;
+
+        LOG_TRC("Anonymizing [" << plain << "] -> [" << anonymized << "].");
+
+        std::unique_lock<std::mutex> lock(AnonymizedMutex);
+
+        AnonymizedStrings[plain] = anonymized;
+    }
+
+    std::string anonymize(const std::string& text)
+    {
+        {
+            std::unique_lock<std::mutex> lock(AnonymizedMutex);
+
+            const auto it = AnonymizedStrings.find(text);
+            if (it != AnonymizedStrings.end())
+            {
+                LOG_TRC("Found anonymized [" << text << "] -> [" << it->second << "].");
+                return it->second;
+            }
+        }
+
+        // We just need something irreversible, short, and
+        // quite simple.
+        std::size_t hash = 0;
+        for (const char c : text)
+            hash += c;
+
+        // Generate the anonymized string. The '#' is to hint that it's anonymized.
+        // Prepend with salt to make it unique, in case we get collisions (which we will, eventually).
+        const std::string res = '#' + Util::encodeId(AnonymizationSalt++, 0) + '#' + Util::encodeId(hash, 0) + '#';
+        mapAnonymized(text, res);
+        return res;
+    }
+
+    std::string getFilenameFromURL(const std::string& url)
+    {
+        std::string base;
+        std::string filename;
+        std::string ext;
+        std::string params;
+        std::tie(base, filename, ext, params) = Util::splitUrl(url);
+        return filename;
+    }
+
+    std::string anonymizeUrl(const std::string& url)
+    {
+        std::string base;
+        std::string filename;
+        std::string ext;
+        std::string params;
+        std::tie(base, filename, ext, params) = Util::splitUrl(url);
+
+        return base + Util::anonymize(filename) + ext + params;
+    }
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
