@@ -21,8 +21,11 @@
 #include <Poco/Net/WebSocket.h>
 #include <Poco/StringTokenizer.h>
 #include <Poco/URI.h>
+#include <Poco/BinaryReader.h>
+#include <Poco/Base64Decoder.h>
 
 #include <common/FileUtil.hpp>
+#include <common/JsonUtil.hpp>
 #include "KitHelper.hpp"
 #include <Log.hpp>
 #include <Png.hpp>
@@ -40,12 +43,13 @@ std::recursive_mutex ChildSession::Mutex;
 
 ChildSession::ChildSession(const std::string& id,
                            const std::string& jailId,
-                           IDocumentManager& docManager) :
+                           DocumentManagerInterface& docManager) :
     Session("ToMaster-" + id, id, false),
     _jailId(jailId),
     _docManager(docManager),
     _viewId(-1),
-    _isDocLoaded(false)
+    _isDocLoaded(false),
+    _copyToClipboard(false)
 {
     LOG_INF("ChildSession ctor [" << getName() << "].");
 }
@@ -118,40 +122,40 @@ bool ChildSession::_handleInput(const char *buffer, int length)
         // Invalidate if we have to
         // TODO instead just a "_invalidate" flag, we should remember / grow
         // the rectangle to invalidate; invalidating everything is sub-optimal
-        if (_stateRecorder._invalidate)
+        if (_stateRecorder.isInvalidate())
         {
             std::string payload = "0, 0, " + std::to_string(INT_MAX) + ", " + std::to_string(INT_MAX) + ", " + std::to_string(curPart);
             loKitCallback(LOK_CALLBACK_INVALIDATE_TILES, payload);
         }
 
-        for (const auto& viewPair : _stateRecorder._recordedViewEvents)
+        for (const auto& viewPair : _stateRecorder.getRecordedViewEvents())
         {
             for (const auto& eventPair : viewPair.second)
             {
                 const RecordedEvent& event = eventPair.second;
-                LOG_TRC("Replaying missed view event: " <<  viewPair.first << " " << LOKitHelper::kitCallbackTypeToString(event._type)
-                                                        << ": " << event._payload);
-                loKitCallback(event._type, event._payload);
+                LOG_TRC("Replaying missed view event: " <<  viewPair.first << " " << LOKitHelper::kitCallbackTypeToString(event.getType())
+                                                        << ": " << event.getPayload());
+                loKitCallback(event.getType(), event.getPayload());
             }
         }
 
-        for (const auto& eventPair : _stateRecorder._recordedEvents)
+        for (const auto& eventPair : _stateRecorder.getRecordedEvents())
         {
             const RecordedEvent& event = eventPair.second;
-            LOG_TRC("Replaying missed event: " << LOKitHelper::kitCallbackTypeToString(event._type) << ": " << event._payload);
-            loKitCallback(event._type, event._payload);
+            LOG_TRC("Replaying missed event: " << LOKitHelper::kitCallbackTypeToString(event.getType()) << ": " << event.getPayload());
+            loKitCallback(event.getType(), event.getPayload());
         }
 
-        for (const auto& pair : _stateRecorder._recordedStates)
+        for (const auto& pair : _stateRecorder.getRecordedStates())
         {
             LOG_TRC("Replaying missed state-change: " << pair.second);
             loKitCallback(LOK_CALLBACK_STATE_CHANGED, pair.second);
         }
 
-        for (const auto& event : _stateRecorder._recordedEventsVector)
+        for (const auto& event : _stateRecorder.getRecordedEventsVector())
         {
-            LOG_TRC("Replaying missed event (part of sequence): " << LOKitHelper::kitCallbackTypeToString(event._type) << ": " << event._payload);
-            loKitCallback(event._type, event._payload);
+            LOG_TRC("Replaying missed event (part of sequence): " << LOKitHelper::kitCallbackTypeToString(event.getType()) << ": " << event.getPayload());
+            loKitCallback(event.getType(), event.getPayload());
         }
 
         _stateRecorder.clear();
@@ -213,6 +217,12 @@ bool ChildSession::_handleInput(const char *buffer, int length)
     {
         assert(false && "Tile traffic should go through the DocumentBroker-LoKit WS.");
     }
+    else if (tokens[0] == "requestloksession" ||
+             tokens[0] == "canceltiles")
+    {
+        // Just ignore these.
+        // FIXME: We probably should do something for "canceltiles" at least?
+    }
     else
     {
         // All other commands are such that they always require a LibreOfficeKitDocument session,
@@ -238,7 +248,9 @@ bool ChildSession::_handleInput(const char *buffer, int length)
                tokens[0] == "saveas" ||
                tokens[0] == "useractive" ||
                tokens[0] == "userinactive" ||
-               tokens[0] == "windowcommand");
+               tokens[0] == "windowcommand" ||
+               tokens[0] == "asksignaturestatus" ||
+               tokens[0] == "signdocument");
 
         if (tokens[0] == "clientzoom")
         {
@@ -324,6 +336,14 @@ bool ChildSession::_handleInput(const char *buffer, int length)
         {
             sendWindowCommand(buffer, length, tokens);
         }
+        else if (tokens[0] == "signdocument")
+        {
+            signDocumentContent(buffer, length, tokens);
+        }
+        else if (tokens[0] == "asksignaturestatus")
+        {
+            askSignatureStatus(buffer, length, tokens);
+        }
         else
         {
             assert(false && "Unknown command token.");
@@ -346,31 +366,33 @@ bool ChildSession::loadDocument(const char * /*buffer*/, int /*length*/, const s
     parseDocOptions(tokens, part, timestamp);
 
     std::string renderOpts;
-    if (!_docOptions.empty())
+    if (!getDocOptions().empty())
     {
         Parser parser;
-        Poco::Dynamic::Var var = parser.parse(_docOptions);
+        Poco::Dynamic::Var var = parser.parse(getDocOptions());
         Object::Ptr object = var.extract<Object::Ptr>();
         Poco::Dynamic::Var rendering = object->get("rendering");
         if (!rendering.isEmpty())
             renderOpts = rendering.toString();
     }
 
-    assert(!_docURL.empty());
-    assert(!_jailedFilePath.empty());
+    assert(!getDocURL().empty());
+    assert(!getJailedFilePath().empty());
 
     std::unique_lock<std::recursive_mutex> lock(Mutex);
 
-    bool loaded = _docManager.onLoad(getId(), _jailedFilePath, _userName,
-            _docPassword, renderOpts, _haveDocPassword, _lang, _watermarkText);
+    const bool loaded = _docManager.onLoad(getId(), getJailedFilePath(), getJailedFilePathAnonym(),
+                                           getUserName(), getUserNameAnonym(),
+                                           getDocPassword(), renderOpts, getHaveDocPassword(),
+                                           getLang(), getWatermarkText());
     if (!loaded || _viewId < 0)
     {
-        LOG_ERR("Failed to get LoKitDocument instance.");
+        LOG_ERR("Failed to get LoKitDocument instance for [" << getJailedFilePathAnonym() << "].");
         return false;
     }
 
     LOG_INF("Created new view with viewid: [" << _viewId << "] for username: [" <<
-            _userName << "] in session: [" << getId() << "].");
+            getUserNameAnonym() << "] in session: [" << getId() << "].");
 
     std::unique_lock<std::mutex> lockLokDoc(_docManager.getDocumentMutex());
 
@@ -508,7 +530,7 @@ void insertUserNames(const std::map<int, UserInfo>& viewInfo, std::string& json)
                 int viewId = action->getValue<int>("viewId");
                 auto it = viewInfo.find(viewId);
                 if (it != viewInfo.end())
-                    action->set("userName", Poco::Dynamic::Var(it->second.Username));
+                    action->set("userName", Poco::Dynamic::Var(it->second.getUserName()));
             }
         }
     }
@@ -647,6 +669,9 @@ bool ChildSession::downloadAs(const char* /*buffer*/, int /*length*/, const std:
         return false;
     }
 
+    // Obfuscate the new name.
+    Util::mapAnonymized(Util::getFilenameFromURL(name), _docManager.getObfuscatedFileId());
+
     getTokenString(tokens[3], "format", format);
 
     if (getTokenString(tokens[4], "options", filterOptions))
@@ -662,18 +687,20 @@ bool ChildSession::downloadAs(const char* /*buffer*/, int /*length*/, const std:
     // Prevent user inputting anything funny here.
     // A "name" should always be a name, not a path
     const Poco::Path filenameParam(name);
-    const auto url = JAILED_DOCUMENT_ROOT + tmpDir + "/" + filenameParam.getFileName();
+    const std::string url = JAILED_DOCUMENT_ROOT + tmpDir + "/" + filenameParam.getFileName();
+    const std::string nameAnonym = anonymizeUrl(name);
+    const std::string urlAnonym = JAILED_DOCUMENT_ROOT + tmpDir + "/" + Poco::Path(nameAnonym).getFileName();
 
     {
         std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
 
-        LOG_DBG("Calling LOK's downloadAs with: '" << url.c_str() << "', '" <<
-                (format.size() == 0 ? "(nullptr)" : format.c_str()) << "', '" <<
-                (filterOptions.size() == 0 ? "(nullptr)" : filterOptions.c_str()) << "'.");
+        LOG_DBG("Calling LOK's downloadAs with: url='" << urlAnonym << "', format='" <<
+                (format.empty() ? "(nullptr)" : format.c_str()) << "', ' filterOptions=" <<
+                (filterOptions.empty() ? "(nullptr)" : filterOptions.c_str()) << "'.");
 
         getLOKitDocument()->saveAs(url.c_str(),
-                format.size() == 0 ? nullptr :format.c_str(),
-                filterOptions.size() == 0 ? nullptr : filterOptions.c_str());
+                                   format.empty() ? nullptr : format.c_str(),
+                                   filterOptions.empty() ? nullptr : filterOptions.c_str());
     }
 
     sendTextFrame("downloadas: jail=" + _jailId + " dir=" + tmpDir + " name=" + name +
@@ -687,6 +714,22 @@ bool ChildSession::getChildId()
     return true;
 }
 
+std::string ChildSession::getTextSelectionInternal(const std::string& mimeType)
+{
+    char* textSelection = nullptr;
+    {
+        std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+
+        getLOKitDocument()->setView(_viewId);
+
+        textSelection = getLOKitDocument()->getTextSelection(mimeType.c_str(), nullptr);
+    }
+
+    std::string str(textSelection ? textSelection : "");
+    free(textSelection);
+    return str;
+}
+
 bool ChildSession::getTextSelection(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
 {
     std::string mimeType;
@@ -698,17 +741,7 @@ bool ChildSession::getTextSelection(const char* /*buffer*/, int /*length*/, cons
         return false;
     }
 
-    char* textSelection = nullptr;
-    {
-        std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
-
-        getLOKitDocument()->setView(_viewId);
-
-        textSelection = getLOKitDocument()->getTextSelection(mimeType.c_str(), nullptr);
-    }
-
-    sendTextFrame("textselectioncontent: " + std::string(textSelection));
-    free(textSelection);
+    sendTextFrame("textselectioncontent: " + getTextSelectionInternal(mimeType));
     return true;
 }
 
@@ -748,20 +781,27 @@ bool ChildSession::insertFile(const char* /*buffer*/, int /*length*/, const std:
         return false;
     }
 
-    if (type == "graphic")
+    if (type == "graphic" || type == "graphicurl")
     {
-        std::string fileName = "file://" + std::string(JAILED_DOCUMENT_ROOT) + "insertfile/" + name;
+        std::string url;
+        if (type == "graphic")
+            url = "file://" + std::string(JAILED_DOCUMENT_ROOT) + "insertfile/" + name;
+        else if (type == "graphicurl")
+            URI::decode(name, url);
+
         std::string command = ".uno:InsertGraphic";
         std::string arguments = "{"
             "\"FileName\":{"
                 "\"type\":\"string\","
-                "\"value\":\"" + fileName + "\""
+                "\"value\":\"" + url + "\""
             "}}";
 
         std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
 
         getLOKitDocument()->setView(_viewId);
 
+        LOG_TRC("Inserting graphic: '" << arguments.c_str() << "', '");
+
         getLOKitDocument()->postUnoCommand(command.c_str(), arguments.c_str(), false);
     }
 
@@ -942,13 +982,19 @@ bool ChildSession::unoCommand(const char* /*buffer*/, int /*length*/, const std:
 
     getLOKitDocument()->setView(_viewId);
 
-    if (tokens.size() == 2 && tokens[1] == ".uno:fakeDiskFull")
+    if (tokens.size() == 2)
     {
-        Util::alertAllUsers("internal", "diskfull");
-    }
-    else if (tokens.size() == 2)
-    {
-        getLOKitDocument()->postUnoCommand(tokens[1].c_str(), nullptr, bNotify);
+        if (tokens[1] == ".uno:fakeDiskFull")
+        {
+            Util::alertAllUsers("internal", "diskfull");
+        }
+        else
+        {
+            if (tokens[1] == ".uno:Copy")
+                _copyToClipboard = true;
+
+            getLOKitDocument()->postUnoCommand(tokens[1].c_str(), nullptr, bNotify);
+        }
     }
     else
     {
@@ -999,6 +1045,7 @@ bool ChildSession::renderWindow(const char* /*buffer*/, int /*length*/, const st
 
     int startX = 0, startY = 0;
     int bufferWidth = 800, bufferHeight = 600;
+    double dpiScale = 1.0;
     std::string paintRectangle;
     if (tokens.size() > 2 && getTokenString(tokens[2], "rectangle", paintRectangle))
     {
@@ -1007,6 +1054,14 @@ bool ChildSession::renderWindow(const char* /*buffer*/, int /*length*/, const st
         startY = std::atoi(rectParts[1].c_str());
         bufferWidth = std::atoi(rectParts[2].c_str());
         bufferHeight = std::atoi(rectParts[3].c_str());
+
+        std::string dpiScaleString;
+        if (tokens.size() > 3 && getTokenString(tokens[3], "dpiscale", dpiScaleString))
+        {
+            dpiScale = std::stod(dpiScaleString);
+            if (dpiScale < 0.001)
+                dpiScale = 1.0;
+        }
     }
     else
         LOG_WRN("windowpaint command doesn't specify a rectangle= attribute.");
@@ -1016,11 +1071,12 @@ bool ChildSession::renderWindow(const char* /*buffer*/, int /*length*/, const st
     int width = bufferWidth, height = bufferHeight;
     std::string response;
     Timestamp timestamp;
-    getLOKitDocument()->paintWindow(winId, pixmap.data(), startX, startY, width, height);
+    getLOKitDocument()->paintWindow(winId, pixmap.data(), startX, startY, width, height, dpiScale);
     const double area = width * height;
     LOG_TRC("paintWindow for " << winId << " returned " << width << "X" << height
             << "@(" << startX << "," << startY << ")"
-            << "and rendered in " << (timestamp.elapsed()/1000.)
+            << " with dpi scale: " << dpiScale
+            << " and rendered in " << (timestamp.elapsed()/1000.)
             << "ms (" << area / (timestamp.elapsed()) << " MP/s).");
 
     response = "windowpaint: id=" + tokens[1] +
@@ -1071,6 +1127,110 @@ bool ChildSession::sendWindowCommand(const char* /*buffer*/, int /*length*/, con
     return true;
 }
 
+namespace
+{
+
+std::string extractCertificate(const std::string & certificate)
+{
+    const std::string header("-----BEGIN CERTIFICATE-----");
+    const std::string footer("-----END CERTIFICATE-----");
+
+    std::string result;
+
+    size_t pos1 = certificate.find(header);
+    if (pos1 == std::string::npos)
+        return result;
+
+    size_t pos2 = certificate.find(footer, pos1 + 1);
+    if (pos2 == std::string::npos)
+        return result;
+
+    pos1 = pos1 + std::string(header).length();
+    pos2 = pos2 - pos1;
+
+    return certificate.substr(pos1, pos2);
+}
+
+std::string extractPrivateKey(const std::string & privateKey)
+{
+    const std::string header("-----BEGIN PRIVATE KEY-----");
+    const std::string footer("-----END PRIVATE KEY-----");
+
+    std::string result;
+
+    size_t pos1 = privateKey.find(header);
+    if (pos1 == std::string::npos)
+        return result;
+
+    size_t pos2 = privateKey.find(footer, pos1 + 1);
+    if (pos2 == std::string::npos)
+        return result;
+
+    pos1 = pos1 + std::string(header).length();
+    pos2 = pos2 - pos1;
+
+    return privateKey.substr(pos1, pos2);
+}
+
+std::vector<unsigned char> decodeBase64(const std::string & inputBase64)
+{
+    std::istringstream stream(inputBase64);
+    Poco::Base64Decoder base64Decoder(stream);
+    std::istreambuf_iterator<char> eos;
+    return std::vector<unsigned char>(std::istreambuf_iterator<char>(base64Decoder), eos);
+}
+
+}
+
+bool ChildSession::signDocumentContent(const char* buffer, int length, const std::vector<std::string>& /*tokens*/)
+{
+    bool bResult = true;
+
+    const std::string firstLine = getFirstLine(buffer, length);
+    const char* data = buffer + firstLine.size() + 1;
+    const int size = length - firstLine.size() - 1;
+    std::string json(data, size);
+
+    Poco::JSON::Parser parser;
+    Poco::JSON::Object::Ptr root = parser.parse(json).extract<Poco::JSON::Object::Ptr>();
+
+    for (auto& chainPtr : *root->getArray("chain"))
+    {
+        assert(chainPtr.isString());
+        std::string chainCertificate = chainPtr;
+        std::vector<unsigned char> binaryChainCertificate = decodeBase64(extractCertificate(chainCertificate));
+
+        bResult = getLOKitDocument()->addCertificate(
+            binaryChainCertificate.data(),
+            binaryChainCertificate.size());
+
+        if (!bResult)
+            return false;
+    }
+
+    std::string x509Certificate = JsonUtil::getJSONValue<std::string>(root, "x509Certificate");
+    std::vector<unsigned char> binaryCertificate = decodeBase64(extractCertificate(x509Certificate));
+
+    std::string privateKey = JsonUtil::getJSONValue<std::string>(root, "privateKey");
+    std::vector<unsigned char> binaryPrivateKey = decodeBase64(extractPrivateKey(privateKey));
+
+    bResult = getLOKitDocument()->insertCertificate(
+                    binaryCertificate.data(), binaryCertificate.size(),
+                    binaryPrivateKey.data(), binaryPrivateKey.size());
+
+    return bResult;
+}
+
+bool ChildSession::askSignatureStatus(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& /*tokens*/)
+{
+    std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
+
+    int nStatus = getLOKitDocument()->getSignatureState();
+
+    sendTextFrame("signaturestatus: " + std::to_string(nStatus));
+    return true;
+}
+
 bool ChildSession::selectGraphic(const char* /*buffer*/, int /*length*/, const std::vector<std::string>& tokens)
 {
     int type, x, y;
@@ -1156,21 +1316,24 @@ bool ChildSession::saveAs(const char* /*buffer*/, int /*length*/, const std::vec
     {
         std::unique_lock<std::mutex> lock(_docManager.getDocumentMutex());
 
-        getLOKitDocument()->setView(_viewId);
-
         if (filterOptions.empty() && format == "html")
         {
             // Opt-in to avoid linked images, those would not leave the chroot.
             filterOptions = "EmbedImages";
         }
 
-        LOG_DBG("Calling LOK's saveAs with: '" << url.c_str() << "', '" <<
+        // We don't have the FileId at this point, just a new filename to save-as.
+        // So here the filename will be obfuscated with some hashing, which later will
+        // get a proper FileId that we will use going forward.
+        LOG_DBG("Calling LOK's saveAs with: '" << anonymizeUrl(wopiFilename) << "', '" <<
                 (format.size() == 0 ? "(nullptr)" : format.c_str()) << "', '" <<
                 (filterOptions.size() == 0 ? "(nullptr)" : filterOptions.c_str()) << "'.");
 
+        getLOKitDocument()->setView(_viewId);
+
         success = getLOKitDocument()->saveAs(url.c_str(),
-                format.size() == 0 ? nullptr :format.c_str(),
-                filterOptions.size() == 0 ? nullptr : filterOptions.c_str());
+                                             format.empty() ? nullptr : format.c_str(),
+                                             filterOptions.empty() ? nullptr : filterOptions.c_str());
 
         if (!success)
         {
@@ -1187,6 +1350,10 @@ bool ChildSession::saveAs(const char* /*buffer*/, int /*length*/, const std::vec
 
             if (retry)
             {
+                LOG_DBG("Retry: calling LOK's saveAs with: '" << url.c_str() << "', '" <<
+                        (format.size() == 0 ? "(nullptr)" : format.c_str()) << "', '" <<
+                        (filterOptions.size() == 0 ? "(nullptr)" : filterOptions.c_str()) << "'.");
+
                 success = getLOKitDocument()->saveAs(url.c_str(),
                         format.size() == 0 ? nullptr :format.c_str(),
                         filterOptions.size() == 0 ? nullptr : filterOptions.c_str());
@@ -1304,30 +1471,28 @@ void ChildSession::rememberEventsForInactiveUser(const int type, const std::stri
     }
 }
 
-void ChildSession::updateSpeed()
-{
+void ChildSession::updateSpeed() {
+
     std::chrono::steady_clock::time_point now(std::chrono::steady_clock::now());
 
-    while (_cursorInvalidatedEvent.size() != 0 &&
-           std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
+    while(_cursorInvalidatedEvent.size() != 0 &&
+        std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
     {
         _cursorInvalidatedEvent.pop();
     }
-
     _cursorInvalidatedEvent.push(now);
     _docManager.updateEditorSpeeds(_viewId, _cursorInvalidatedEvent.size());
 }
 
-int ChildSession::getSpeed()
-{
+int ChildSession::getSpeed() {
+
     std::chrono::steady_clock::time_point now(std::chrono::steady_clock::now());
 
-    while (_cursorInvalidatedEvent.size() > 0 &&
-           std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
+    while(_cursorInvalidatedEvent.size() > 0 &&
+        std::chrono::duration_cast<std::chrono::milliseconds>(now - _cursorInvalidatedEvent.front()).count() > _eventStorageIntervalMs)
     {
         _cursorInvalidatedEvent.pop();
     }
-
     return _cursorInvalidatedEvent.size();
 }
 
@@ -1359,7 +1524,7 @@ void ChildSession::loKitCallback(const int type, const std::string& payload)
         }
     }
 
-    switch (type)
+    switch (static_cast<LibreOfficeKitCallbackType>(type))
     {
     case LOK_CALLBACK_INVALIDATE_TILES:
         {
@@ -1536,8 +1701,36 @@ void ChildSession::loKitCallback(const int type, const std::string& payload)
     case LOK_CALLBACK_VALIDITY_LIST_BUTTON:
         sendTextFrame("validitylistbutton: " + payload);
         break;
+    case LOK_CALLBACK_CLIPBOARD_CHANGED:
+    {
+        std::string selection;
+        if (_copyToClipboard)
+        {
+            _copyToClipboard = false;
+            selection = getTextSelectionInternal("");
+        }
+
+        sendTextFrame("clipboardchanged: " + selection);
+        break;
+    }
+    case LOK_CALLBACK_CONTEXT_CHANGED:
+        sendTextFrame("context: " + payload);
+        break;
+    case LOK_CALLBACK_SIGNATURE_STATUS:
+        sendTextFrame("signaturestatus: " + payload);
+        break;
+
+    case LOK_CALLBACK_DOCUMENT_PASSWORD:
+    case LOK_CALLBACK_DOCUMENT_PASSWORD_TO_MODIFY:
+        // these are not handled here.
+        break;
+
+#if !ENABLE_DEBUG
+    // we want a compilation-time failure in the debug builds; but ERR in the
+    // log in the release ones
     default:
         LOG_ERR("Unknown callback event (" << type << "): " << payload);
+#endif
     }
 }
 
