@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
  * L.Control is a base class for implementing map controls. Handles positioning.
  * All other controls extend from this class.
@@ -129,7 +130,6 @@ L.Map.include({
 		}
 
 		createCorner('top', 'left');
-		createCorner('top', 'middle');
 		createCorner('top', 'right');
 		createCorner('bottom', 'left');
 		createCorner('bottom', 'right');
