@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
  * L.WOPI contains WOPI related logic
  */
@@ -19,6 +20,8 @@ L.Map.WOPI = L.Handler.extend({
 	DisableCopy: false,
 	DisableInactiveMessages: false,
 	UserCanNotWriteRelative: true,
+	EnableInsertRemoteImage: false,
+	EnableShare: false,
 	CallPythonScriptSource: null,
 
 	_appLoadedConditions: {
@@ -74,6 +77,8 @@ L.Map.WOPI = L.Handler.extend({
 		this.DisableCopy = !!wopiInfo['DisableCopy'];
 		this.DisableInactiveMessages = !!wopiInfo['DisableInactiveMessages'];
 		this.UserCanNotWriteRelative = !!wopiInfo['UserCanNotWriteRelative'];
+		this.EnableInsertRemoteImage = !!wopiInfo['EnableInsertRemoteImage'];
+		this.EnableShare = !!wopiInfo['EnableShare'];
 
 		this._map.fire('postMessage', {
 			msgId: 'App_LoadingStatus',
@@ -176,6 +181,29 @@ L.Map.WOPI = L.Handler.extend({
 				}
 			}
 		}
+		if (msg.MessageId === 'Show_Button' || msg.MessageId === 'Hide_Button') {
+			if (!msg.Values) {
+				console.error('Property "Values" not set');
+				return;
+			}
+			if (!msg.Values.id) {
+				console.error('Property "Values.id" not set');
+				return;
+			}
+			if (this._map._permission !== 'edit') {
+				console.log('No toolbar in readonly mode - ignoring Remove_Button request.');
+				return;
+			}
+			if (!w2ui['toolbar-up'].get(msg.Values.id)) {
+				console.error('Toolbar button with id "' + msg.Values.id + '" not found.');
+				return;
+			}
+			if (msg.MessageId === 'Show_Button') {
+				w2ui['toolbar-up'].show(msg.Values.id);
+			} else {
+				w2ui['toolbar-up'].hide(msg.Values.id);
+			}
+		}
 		else if (msg.MessageId === 'Set_Settings') {
 			if (msg.Values) {
 				var alwaysActive = msg.Values.AlwaysActive;
@@ -215,6 +243,11 @@ L.Map.WOPI = L.Handler.extend({
 				this._map.downloadAs(fileName + '.' + format, format);
 			}
 		}
+		else if (msg.MessageId == 'Action_InsertGraphic') {
+			if (msg.Values) {
+				this._map.insertURL(msg.Values.url);
+			}
+		}
 		else if (msg.MessageId === 'Action_ShowBusy') {
 			if (msg.Values && msg.Values.Label) {
 				this._map.fire('showbusy', {label: msg.Values.Label});
