@@ -25,6 +25,7 @@
 #include <Log.hpp>
 #include <Unit.hpp>
 #include <Util.hpp>
+#include <wsd/LOOLWSD.hpp>
 
 void Document::addView(const std::string& sessionId, const std::string& userName, const std::string& userId)
 {
@@ -88,7 +89,7 @@ const std::string Document::getHistory() const
     std::ostringstream oss;
     oss << "{";
     oss << "\"docKey\"" << ":\"" << _docKey << "\",";
-    oss << "\"filename\"" << ":\"" << getFilename() << "\",";
+    oss << "\"filename\"" << ":\"" << LOOLWSD::anonymizeUrl(getFilename()) << "\",";
     oss << "\"start\"" << ":" << _start << ",";
     oss << "\"end\"" << ":" << _end << ",";
     oss << "\"pid\"" << ":" << getPid() << ",";
@@ -178,7 +179,7 @@ void AdminModel::assertCorrectThread() const
 
 AdminModel::~AdminModel()
 {
-    LOG_DBG("History:\n\n" << getAllHistory() << '\n');
+    LOG_TRC("History:\n\n" << getAllHistory() << '\n');
     LOG_INF("AdminModel dtor.");
 }
 
@@ -665,6 +666,7 @@ std::string AdminModel::getDocuments() const
             Poco::URI::encode(it.second.getFilename(), " ", encodedFilename);
             oss << separator1 << '{' << ' '
                 << "\"pid\"" << ':' << it.second.getPid() << ','
+                << "\"docKey\"" << ':' << '"' << it.second.getDocKey() << '"' << ','
                 << "\"fileName\"" << ':' << '"' << encodedFilename << '"' << ','
                 << "\"activeViews\"" << ':' << it.second.getActiveViews() << ','
                 << "\"memory\"" << ':' << it.second.getMemoryDirty() << ','
